; This RS232 receiver works by shifting bits in/out of a register till a start bit comes out into CARRY
; once that happens, we know we have 8 bits in the register. This is experemental and I don't know if it
; will even work. Receiver samples the start bit a 2x Baud rate and should be accurate at low speeds.
; we implement a protocol where each byte would be:
; hnnnxxxx where h-hi/lo nibble flag,  nnn-pwm number, xxxx - nibble of pwm data
; if nn is 11, that is direction nibble that gets saved in direction variable
; baud rate	pulse time
; 110		9.09ms
; 300		3.33ms
; 1200		833us
; 2400		417us
; 4800		208us
; 9600		104us
; speed(Mhz)=baud*4(hzpercycle)*( readbit(10)+x*pwm(36)*2(2delays per cycle) )
; 3,579,545=300*4*(10+36*2*x), x=42 (41.291)
; at these values pwm cycles happen at 100hz ( each pulse takes 1/100s )


	list      p=16F84             ; list directive to define processor
	#include <p16F84.inc>         ; processor specific variable definitions
	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _HS_OSC

TRISAVAL		EQU	B'10000'		; RA4 is input - SERIAL_PIN. Lower nibble is H-Bridge C
;;;TRISAVAL		EQU	B'00001'		;;; DEBUG ONLY !!!
TRISBVAL		EQU	B'00000000'	; H-Bridge A and B
SERIAL_PORT	EQU	PORTA		;
DELAY_LOOPS	EQU	D'42'		; how many times to loop in delay procedure
HINIB_FLAG	EQU	7			; HI / LO nibble bit in the protocol

 CBLOCK 0C
	pwm_cycle		; duty cycle counter for pwm - does not need to be initialized
	pwm_state		; used in do_pwm as temp value for PORTB
	shiftreg		; shift register - contains received data
	delay_counter	; used in delay routine
	pwm0,pwm1,pwm2,pwm3,pwm4,pwm5,dirflags	; array of values set by communication protocol
 ENDC

DOPWM	MACRO	pwm,pin	; used in do_pwm - 4 cycles
	movf pwm,W		; load pwm value from array of pwms: pwm0-pwm5
	subwf pwm_cycle,W	; pwm0 - pwm_cycle ( pwm0>pwm_cycle? )
	btfsc STATUS,C		; is it time to turn the duty cycle off?
	bcf pwm_state,pin	; set bit lo
	ENDM


	ORG 0			; NO INTERRUPTS :)
	clrf PORTA		; init port A
	clrf PORTB		; init port B
	bsf STATUS,RP0		; select bank 1
	movlw TRISAVAL
	movwf TRISA
	movlw TRISBVAL
	movwf TRISB
	bcf STATUS,RP0		; most regs are in bank 0
	clrf pwm0
	clrf pwm1
	clrf pwm2
	clrf pwm3
	clrf pwm4
	clrf pwm5
	clrf dirflags
	clrf shiftreg		; init shiftreg
read_bit:				; 11 instruction RS232 protocol code anyone?
	call do_delay		; while waiting for a start bit sample at twice the speed
	bcf STATUS,C		; prepare to set carry
	btfsc SERIAL_PORT,4	; check if data/start bit is hi or lo
;;;	btfsc SERIAL_PORT,0	;;; DEBUG ONLY!!!
	bsf STATUS,C		; if input on serial port is 1, set carry
	rrf shiftreg,F		; move serial bit(CARRY bit) into shiftreg
	btfsc STATUS,C		; was start bit carried out?
	goto finished_byte	; otherwise see if we are reading data and should delay
	movf shiftreg,F	; set ZERO flag ( if shiftreg != 0 then at least start bit is in shiftreg )
	btfss STATUS,Z		; delay twice if reading the byte and not just waiting for a start bit
	call do_delay		; two delays per bit
	goto read_bit		; till rlf puts a 1 in a CARRY, we are not done reading 8 bits
finished_byte:			; process data here
	movlw b'11111111'	; RS232 has it backwards - invert the data
	xorwf shiftreg,F	; logical 1 is -12v and 0 is +12v. we get 0v=1,5v=0
	swapf shiftreg,W	; save pwm nibble using indirect addressing into pwmX
	andlw b'00000111'	; get the address of pwmX (pwm0 through pwm5 + direction)
	addlw pwm0		; add it to the base array address
	movwf FSR			; save address of pwmX or direction flags
	btfsc shiftreg, HINIB_FLAG	; see if it is a hi or low nibble
	swapf INDF,F		; prepare to fill hi nibble in low nibble place
	movlw b'11110000'	; clear lower nibble
	andwf INDF,F		; so that we can put the new data in
	movlw b'00001111'	; only lower 4 bits from shift reg are used as a pwm value
	andwf shiftreg,W	; so load the lower nibble of serial data into W register
	iorwf INDF,F		; combine the lower and upper nibble
	btfsc shiftreg, HINIB_FLAG	; see if nibbles were swapped before
	swapf INDF,F		; now swap them back
	call do_delay		; delay again after 8 bits are read
	clrf shiftreg		; reinit shiftreg
	goto read_bit		;



do_delay:
	movlw DELAY_LOOPS	; load the delay
	movwf delay_counter	; store the delay in DELAY
delay_loop:			; do one step of pulse width modulation
	incf pwm_cycle,F	; 100% duty cycle counter - 256 time units
	btfss STATUS, Z	; just starting a new cycle?
	goto continue_cycle	; no, we are continuing a cycle
	clrf PORTA		; not setting pins instanteneously will create a small delay and
	clrf PORTB		; prevent shoot - through currents in H-bridges
	movf dirflags,W	; load directions nibble
	movwf PORTA		; set direction pins on PORTA (pin4 is output and not affected)
	btfsc dirflags,4	; set direction bits 6 and 7 in pwm_state 
	bsf PORTB,6
	btfsc dirflags,5
	bsf PORTB,7
continue_cycle:
	movlw B'00111111'	; for now, all lines are ON - this walue will go into pwm_state
	iorwf PORTB,W		; bits 6 and 7 are direction bits and change only during start of cycle
	movwf pwm_state	; DOPWM operates on pwm_state
	DOPWM pwm0,0		; macro to set or clear pwm_state bits depending on pwmX values
	DOPWM pwm1,1		; pwmX and pin# that it corresponds to
	DOPWM pwm2,2		; 4 cycles per macro
	DOPWM pwm3,3
	DOPWM pwm4,4
	DOPWM pwm5,5
	movf pwm_state,W	; now that we have it all set, set the PWM pins on PORTB to those values
	movwf PORTB
	decfsz delay_counter,F
	goto delay_loop	; loop while delay is not 0
	return


 END
