#include <windows.h>
#include <stdio.h>
#include "ComPort.h"

// byte format: hnnnxxxx where xxxx - pwm data, nnn - channel #, h - hi nibble flag
#define CH0 0
#define CH1 1
#define CH2	2
#define CH3	3
#define CH4	4
#define CH5	5
#define DIRBITS 6
#define DIR0	1
#define DIR1	2
#define DIR2	4
#define DIR3	8
#define DIR4	0x10
#define DIR5	0x20
#define HIBIT	0x80
#define SCHMITT_TRIGGER 800	// 600

ComPort	com;


void write(unsigned char channel, unsigned char data){
	channel = (channel&0x7)<<4; // mask lower 3 bits and move them in upper nibble
	unsigned char hi = channel|(data>>4)|HIBIT;
	unsigned char lo = channel|(data&0xF);
	com.Write((char*)&hi,1);
	com.Write((char*)&lo,1);
}

void joy(){
static JOYINFO pji; // UINT wXpos wYpos wZpos wButtons
//static char * bspstr = "\b\b\b\b\b\b\b\b\b\b`\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b";
//static char * spstr = "          ";
static unsigned char oldX, oldY, oldDirbits;
unsigned int rawX, rawY;
unsigned char x,y, dirbits;
	if(JOYERR_NOERROR != joyGetPos(JOYSTICKID1,&pji)){
		printf("Error getting joystick position\n");
		return;
	}
	rawX = abs(pji.wXpos-0x7FFF);
	rawY = abs(pji.wYpos-0x7FFF);
	x = rawX/0x80;
	y = rawY/0x80;
	if(rawX < SCHMITT_TRIGGER){ // joystick is too sensitive
		x = 0;
	}
	if(rawY < SCHMITT_TRIGGER){
		y = 0;
	}
	if(oldX !=x){
		write(CH4,x); // channel 4 (0 based)
	}
	if(oldY !=y){
		write(CH5,y); // channel 5 (0 based)
	}
	dirbits=0;
	dirbits|=(pji.wYpos>0x7FFF-SCHMITT_TRIGGER)?DIR5:0; // schmitt triggered from 7FFF
	dirbits|=(pji.wXpos>0x7FFF+SCHMITT_TRIGGER)?DIR4:0; // schmitt triggered
	if(oldDirbits != dirbits){
		write(DIRBITS,dirbits);
	}
	if(oldX!=x||oldY!=y||oldDirbits!=dirbits){
		printf("rawX= %d, x=%d, rawY=%d, y=%d dirbits=0x%X\n",pji.wXpos,x,pji.wYpos,y,dirbits);
	}
	if(pji.wButtons == JOY_BUTTON1){
		write(DIRBITS,0x0F); // channels 1-4 directions ON
		printf("Button1\n");
	}
	if(pji.wButtons == JOY_BUTTON2){
		write(DIRBITS,0x00); // channels 1-4 directions OFF
		printf("Button2\n");
	}
	oldX = x;
	oldY = y;
	oldDirbits = dirbits;
}


void main(void){
	com.Open(2);
	while(1){
		joy();
//		Sleep(100);
	}
}
